@@ -12,11 +12,13 @@ import sep
 from dask import delayed
 from owl_dev.logging import logger
 
+from ..typing_utils import NumArray
+
 
 def create_mask(
-    image,
+    image: NumArray,
     cl_morph_parm=10,
-    c_min_area=0.01,
+    c_min_area=0.0009,
     c_count_max=15,
     output_size=None,
 ):
@@ -42,9 +44,9 @@ def create_mask(
         Mask image with mask values set to 1 and background values set to 0
     """
 
-    # TODO: why not (image - image.min()) / (image.max() - image.min()) * 255
-    image = (image - image.min()) / image.max() * 255
-    image = np.array(image).astype(np.uint8)
+    # to avoid compressing all background values around 0, and dealing with
+    # zero values for background, we scale data to 25 and and add 1 later
+    image = ((image - image.min()) / (image.max() - image.min()) * 254) + 1.0
 
     # apply close morphology to get rid of small structures and make the big blog bolder
     outer = cv2.morphologyEx(
@@ -53,9 +55,15 @@ def create_mask(
         cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (cl_morph_parm, cl_morph_parm)),
     )
 
+    # fill/replace zeros with median value to have smoother background
+    # then we apply blur filter to remove any artificial casued by filling with median
+    outer[outer <= 1] = np.median(outer[outer > 1])
+    outer = outer.astype(np.uint8)
+    outer = cv2.blur(outer, (5, 5))
+
     # remove noise by filtering the most frequent element from the image
     ret, th1 = cv2.threshold(
-        image, np.median(outer[outer > 0.0]) + 1, 255, cv2.THRESH_BINARY
+        outer, np.median(outer), 255, cv2.THRESH_BINARY + cv2.THRESH_TRIANGLE
     )
 
     # binaries the image and fill the holes
@@ -68,7 +76,7 @@ def create_mask(
     # create an empty mask image
     mask = np.zeros(image.shape, np.uint8)
 
-    # assuming the min area for tissue is (e.g. 0.01 = 0.1x * 0.1y)
+    # assuming the min area for tissue is (e.g. 0.0009 = 0.03x * 0.03y)
     min_blob_area = c_min_area * image.shape[1] * image.shape[0]
     blob_counter = 0
 
@@ -80,9 +88,6 @@ def create_mask(
             cv2.drawContours(mask, [cnt], -1, 255, cv2.FILLED)
             mask = cv2.bitwise_and(image_fill_holes, mask)
 
-    # also mask all pixels where in original image have val=0 (out of field zone)
-    mask[image == 0] = 1.0
-
     if output_size is not None:
         mask = cv2.resize(mask, dsize=output_size, interpolation=cv2.INTER_LINEAR)
 
@@ -97,7 +102,7 @@ def run_sep(
     chunk_loc=None,
     bw=128,
     bh=128,
-    minarea=900,
+    minarea=880,
     deblend_nthresh=2,
     circularity=0.9,
     std_l16=None,
@@ -106,52 +111,60 @@ def run_sep(
     if chunk_loc is None:
         chunk_loc = [[np.nan, np.nan], [np.nan, np.nan]]
 
-    logger.debug("Running SEP on the chunk index = %s", indx)
-
     im = np.array(im)
     mask = np.array(mask).astype(bool)
 
     extc_bkg = sep.Background(im, mask=mask, bw=bw, bh=bh)
 
-    logger.debug("Global Background: %f", extc_bkg.globalback)
-    logger.debug("Global RMS: %f", extc_bkg.globalrms)
+    logger.debug(
+        "Background: %f - Global RMS: %f", extc_bkg.globalback, extc_bkg.globalrms
+    )
 
     # subtract the background
     reduced_data = im - extc_bkg.back()
-    threshold1 = max(0, np.mean(reduced_data[np.invert(mask)])) + (
-        1.5 * np.std(extc_bkg.back())
+    # here you could introduce as many detection-thresholds as you want to give a try
+    threshold_0 = max(0, np.mean(reduced_data[np.invert(mask)])) + (
+        1.3 * np.std(extc_bkg.back())
     )
-    threshold2 = max(0, np.mean(reduced_data[np.invert(mask)])) + (1.5 * std_l16)
+    threshold_1 = max(0, np.mean(reduced_data[np.invert(mask)])) + (
+        1.8 * np.std(extc_bkg.back())
+    )
+    threshold_2 = max(0, np.mean(reduced_data[np.invert(mask)])) + (
+        2.3 * np.std(extc_bkg.back())
+    )
+    threshold_3 = max(0, np.mean(reduced_data[np.invert(mask)])) + (
+        3.0 * np.std(extc_bkg.back())
+    )
+    threshold_4 = max(0, np.mean(reduced_data[np.invert(mask)])) + (1.5 * std_l16)
+    threshold_list = [threshold_0, threshold_1, threshold_2, threshold_3, threshold_4]
 
     try:
-        # detect objects (try 1)
-        objects = sep.extract(
-            reduced_data,
-            threshold1,
-            minarea=minarea,
-            deblend_nthresh=deblend_nthresh,
-            mask=mask,
-            segmentation_map=False,
-        )
+        for i_th, thr_i in enumerate(threshold_list):
+            try:
+                # detect objects (try 1): Using the local (block) bacground
+
+                objects = sep.extract(
+                    reduced_data,
+                    thr_i,
+                    minarea=minarea,
+                    deblend_nthresh=deblend_nthresh,
+                    mask=mask,
+                    segmentation_map=False,
+                )
+                break
+            except Exception:
+                objects = None
+                if i_th < len(threshold_list) - 1:
+                    continue
     except Exception:
-        try:
-            # detect objects (try 2)
-            logger.warning(
-                "Second try: Object detection [deploying a GLOBAL l16 background]"
-            )
-            objects = sep.extract(
-                reduced_data,
-                threshold2,
-                minarea=minarea,
-                deblend_nthresh=deblend_nthresh,
-                mask=mask,
-                segmentation_map=False,
-            )
-        except Exception:
-            exc_type, exc_value, exc_traceback = sys.exc_info()
-            lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
-            logger.debug("".join(lines))
-            return None
+        exc_type, exc_value, exc_traceback = sys.exc_info()
+        lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
+        logger.debug("".join(lines))
+        return None
+
+    if objects is None:
+        logger.debug("No objects found")
+        return pd.DataFrame()
 
     objects_df = pd.DataFrame(objects)
 
@@ -167,30 +180,63 @@ def run_sep(
     objects_df.insert(3, "X_RANGE_f", chunk_loc[1][1])
     objects_df.insert(4, "Y_RANGE_i", chunk_loc[0][0])
     objects_df.insert(5, "Y_RANGE_f", chunk_loc[0][1])
+    objects_df.rename({"x": "x_block", "y": "y_block"}, axis=1, inplace=True)
 
     # estimate absolute coordinates of the peaks and keep them as new columns
-    # called XC and YC (also corrected for overlapping)
+    # called x and y (also corrected for overlapping)
     # Here we use (2 * index +1) * overlap_size factor to convert coordinates
     # from overlapped to the original (non-overlapped)
-    if df_len > 0:
-        objects_df["XC"] = objects_df.apply(
-            lambda row: row.X_RANGE_i + row.x - (2.0 * row.indx_1 + 1) * overlap_size,
-            axis=1,
-        )
-        objects_df["YC"] = objects_df.apply(
-            lambda row: row.Y_RANGE_i + row.y - (2.0 * row.indx_0 + 1) * overlap_size,
-            axis=1,
-        )
-    else:
-        objects_df["XC"] = []
-        objects_df["YC"] = []
 
-    logger.debug("Number of detected objects: %s", df_len)
+    if df_len > 0:
+
+        fn_x = (
+            lambda row: row.X_RANGE_i
+            + row.x_block
+            - (2.0 * row.indx_1 + 1) * overlap_size
+        )
+
+        col_x = objects_df.apply(fn_x, axis=1)
+        objects_df.assign(x=col_x.values)
+
+        fn_y = (
+            lambda row: row.Y_RANGE_i
+            + row.y_block
+            - (2.0 * row.indx_0 + 1) * overlap_size
+        )
+
+        col_y = objects_df.apply(fn_y, axis=1)
+        objects_df.assign(y=col_y.values)
+
+        # TODO: more accurate way to estimate the radius
+        fn_r = (lambda row: np.sqrt(row.a ** 2 + row.b ** 2) / 2)
+        col_r = objects_df.apply(fn_r, axis=1)
+        objects_df.assign(r=col_r.values)
+
+    else:
+        objects_df["x"] = []
+        objects_df["y"] = []
+        objects_df["r"] = []
+
+    logger.debug("Number of detected objects: %s (try: %d)", df_len, i_th + 1)
 
     return objects_df
 
 
-def bead_detect_worker(image, mask, overlap_size=0.0, std_l16=None):
+@delayed
+def compute_background_l16(image_l16: NumArray, mask_l16: NumArray, bw=128, bh=128):
+    try:
+        l16_bkg = sep.Background(image_l16, mask=mask_l16, bw=128, bh=128)
+        std_l16 = np.std(l16_bkg.back())
+    except Exception:
+        std_l16 = np.nanquantile(image_l16[mask_l16 == 0], 0.75)
+    return std_l16
+
+
+def bead_detect_worker(image, mask, image_l16, mask_l16, overlap_size=0.0):
+
+    # Evaluate standard deviation of background extracted from l16
+    # In case the original background failed, we deploy this value
+    std_l16 = compute_background_l16(image_l16, mask_l16, bw=128, bh=128)
 
     if overlap_size > 0.0:
         # add overlap to data and mask (for mask, we refill the boundary with 1 [equivalent to mask=True])
@@ -227,9 +273,11 @@ def bead_detect_worker(image, mask, overlap_size=0.0, std_l16=None):
 
 
 def plot_beads(image, mask, beads_df, fig_fname=None):
+
     try:
         import matplotlib
-        matplotlib.use('Agg')
+
+        matplotlib.use("Agg")
         import matplotlib.pyplot as plt
         from matplotlib.patches import Ellipse
     except ModuleNotFoundError:
@@ -253,7 +301,7 @@ def plot_beads(image, mask, beads_df, fig_fname=None):
         try:
             # plot boarder or each ellipse
             e = Ellipse(
-                xy=(beads_df["XC"][i], beads_df["YC"][i]),
+                xy=(beads_df["x"][i], beads_df["x"][i]),
                 width=6 * beads_df["a"][i],
                 height=6 * beads_df["b"][i],
                 angle=beads_df["theta"][i] * 180.0 / np.pi,
@@ -264,7 +312,7 @@ def plot_beads(image, mask, beads_df, fig_fname=None):
 
             # mark the centre of each ellipse
             e_cent = Ellipse(
-                xy=(beads_df["XC"][i], beads_df["YC"][i]),
+                xy=(beads_df["x"][i], beads_df["x"][i]),
                 width=0.5 * beads_df["a"][i],
                 height=0.5 * beads_df["b"][i],
                 angle=beads_df["theta"][i] * 180.0 / np.pi,
