@@ -3,11 +3,12 @@ import dask.array as da
 import numpy as np
 import xarray as xr
 from dask import delayed
-from imaxt_mosaic.raw import GenericSection, GenericStitcher
-from imaxt_mosaic.settings import Settings
 from owl_dev.logging import logger
 from shapely.geometry import Polygon
 
+from imaxt_mosaic.raw import GenericSection, GenericStitcher
+from imaxt_mosaic.settings import Settings
+
 from ..stagepos import StagePos
 from ..utils import cancel_futures, get_workers_address, persist_array
 from . import register_stitcher
@@ -16,7 +17,7 @@ __config = {
     "bscale": 1,
     "bzero": 0,
     "sections": None,
-    "channel_to_use": 3,
+    "channel_to_use": -1,
     "norm_val": 1,
     "beads": {
         "min_radius": 15,
