@@ -3,9 +3,10 @@ import json
 import dask
 import numpy as np
 import xarray as xr
+from owl_dev.logging import logger
+
 from imaxt_mosaic.raw import GenericSection, GenericStitcher
 from imaxt_mosaic.settings import Settings
-from owl_dev.logging import logger
 
 from ..stagepos import StagePos, get_avg_grid
 from ..utils import cancel_futures, get_workers_address
@@ -17,7 +18,7 @@ __config = {
     "x_min": 0,
     "x_max": 2080,
     "norm_val": 1,
-    "channel_to_use": 3,
+    "channel_to_use": None,
     "scale": 0.55,
     "cof_dist": {
         "cof_x": [
@@ -126,7 +127,9 @@ class Stitcher(GenericStitcher):
                 images = section.calibrate(z, cals, persist=True)
                 if grid is None:
                     avg_grid = get_avg_grid(output_path / "registration")
-                    grid = section.calculate_offsets(images[Settings.channel_to_use], avg_grid=avg_grid)
+                    grid = section.calculate_offsets(
+                        images[Settings.channel_to_use], avg_grid=avg_grid
+                    )
 
                 section.stitch(images, z, grid, output_path / "mos", conf=conf)
                 logger.debug("Stitching done")
